function [clt,mi] = chowLiu(obj,data,h)
% Learn the network structure from training data.
%   data: (#variables,1) cell array containing the training data for each variable.
%   h   : Handle to a function that estimates the entropy of a (dim_{v_i},samples) matrix.
%
% (c)2014 Andreas M. Lehrmann

v = length(data);
mi = zeros(v,v);
for i=1:v
    lDisp(['Computing mutual informations for variable ' int2str(i) '...'])
    for j=i+1:v
        cData = {data{i},data{j}};
        mi(i,j) = h(cData{1})+h(cData{2})-h([cData{1};cData{2}]);
        mi(j,i) = mi(i,j);
    end
end

if any(isnan(mi))
    %lDisp('NaN discovered. Setting to 1e-5.')
    mi(isnan(mi)) = 1e-5;
end
if any(mi<0)
    %lDisp('Negative MI discovered. Setting to 1e-5.')
    mi(mi<0) = 1e-5;
end

assert(obj.isConnected(mi),'Graph of pairwise mutual informations is not connected.')
lDisp('Directing nodes away from the root...')
clt = direct(obj.mst(mi,'max'));

end

function [clt] = direct(clt)

S = 1;
while ~isempty(S)
    current = S(1);
    S(1) = [];
    
    n = find(clt(current,:));
    clt(n,current) = 0;
    S = [S n];
end

end