function all = all(obj, data, cond, coreClusters)
% Compute an approximate log-likelihood.
%   data        : (dim-length(pIndex),1) double array. The data point.
%   cond        : [] or (length(pIndex),1) double array. The condition.
%   coreClusters: Scalar integer between 1 and totalClusters.
%                 The number of exact clusters to use for the computation.
%
% (c)2014 Andreas M. Lehrmann

assert(~isempty(obj.totalClusters),'Approximate log-likelihoods are turned off.')

if isempty(cond)    
    %[~,clusterIndices] = pdist2mex(obj.jointClustering.center,obj.LJ*data,'euc',[],coreClusters,[]); %get closest cluster centers (exhaustive search)
    [~,clusterIndices] = sort(sum((obj.jointClustering.center - obj.LJ*data*ones(1,obj.totalClusters)).^2,1)); %get closest cluster centers (exhaustive search)
    clusterIndices = clusterIndices(1:coreClusters);
    dataIndices = vertcat(obj.jointClustering.inverseId{clusterIndices});
    remaining = 1:obj.totalClusters;
    remaining(clusterIndices) = [];
    
    exact = sum((obj.LJ*data*ones(1,length(dataIndices))-obj.PJ(:,dataIndices)).^2);
    approximate = sum((obj.LJ*data*ones(1,length(remaining))-obj.jointClustering.center(:,remaining)).^2);
    weights=[ones(1,length(dataIndices)) obj.jointClustering.weights(remaining)];
    
    all = obj.constantStatic+logsumwexp(-0.5*([exact approximate]),weights);
else
    joint = [data;cond];
    
    %[~,clusterIndicesJoint] = pdist2mex(obj.jointClustering.center,obj.LJ*joint,'euc',[],coreClusters,[]); %get closest cluster centers (exhaustive search)
    [~,clusterIndicesJoint] = sort(sum((obj.jointClustering.center - obj.LJ*joint*ones(1,obj.totalClusters)).^2,1)); %get closest cluster centers (exhaustive search)
    clusterIndicesJoint = clusterIndicesJoint(1:coreClusters);
    dataIndicesJoint = vertcat(obj.jointClustering.inverseId{clusterIndicesJoint});  
    remainingJoint = 1:obj.totalClusters;
    remainingJoint(clusterIndicesJoint) = [];

    %[~,clusterIndicesMarginal] = pdist2mex(obj.marginalClustering.center,obj.LM*cond,'euc',[],coreClusters,[]); %get closest cluster centers (exhaustive search)
    [~,clusterIndicesMarginal] = sort(sum((obj.marginalClustering.center - obj.LM*cond*ones(1,obj.totalClusters)).^2,1)); %get closest cluster centers (exhaustive search)
    clusterIndicesMarginal = clusterIndicesMarginal(1:coreClusters);
    dataIndicesMarginal = vertcat(obj.marginalClustering.inverseId{clusterIndicesMarginal});  
    remainingMarginal = 1:obj.totalClusters;
    remainingMarginal(clusterIndicesMarginal) = []; 

    all = obj.constantStatic-obj.constantDynamic + ...
          logsumwexp(-0.5*([sum((obj.LJ*joint*ones(1,length(dataIndicesJoint))-obj.PJ(:,dataIndicesJoint)).^2,1) ... %exact joint
                            sum((obj.LJ*joint*ones(1,length(remainingJoint))-obj.jointClustering.center(:,remainingJoint)).^2,1)]), ... %approximate joint
                           [ones(1,length(dataIndicesJoint)) obj.jointClustering.weights(remainingJoint)]) - ... %weights joint
          logsumwexp(-0.5*([sum((obj.LM*cond*ones(1,length(dataIndicesMarginal))-obj.PM(:,dataIndicesMarginal)).^2,1) ... %exact marginal
                            sum((obj.LM*cond*ones(1,length(remainingMarginal))-obj.marginalClustering.center(:,remainingMarginal)).^2,1)]), ... %approximate marginal
                           [ones(1,length(dataIndicesMarginal)) obj.marginalClustering.weights(remainingMarginal)]); %weights marginal
end

end