function [] = estimate(obj, data, pIndex, options)
% Estimate a conditional KDE model from training data.
%   data   : (dim,samples) double array. The training data.
%   pIndex : Dimensions that are being conditioned on, e.g., [4 5 6].
%            Empty for an unconditional KDE model.
%   options: Matlab structure with the following fields:
%     best: The employed technique for KDE bandwidth estimation.
%           Possible values are {'full','diagonal','isotropic'}. Optional with default 'full'.
% 
% (c)2014 Andreas M. Lehrmann              
             
obj.data = data;
obj.pIndex = pIndex;
[obj.dim,obj.samples]=size(data);

if isfield(options,'best') obj.best = options.best;
else obj.best = 'full';
end

f = obj.samples^(-1/(4+obj.dim));
[vec,val] = eig(cov(data')); %cov = vec*val*vec'
val = diag(val);
val(val<1e-4) = 1e-4;
val = diag(val);
obj.B = f*vec*sqrt(val)*vec';
obj.B = (obj.B+obj.B')/2;
if strcmp(obj.best,'diagonal') obj.B = diag(diag(obj.B));end
if strcmp(obj.best,'isotropic') obj.B = (1/size(obj.B,1))*trace(obj.B)*eye(size(obj.B));end
assert(det(obj.B)>0);

obj.B2 = obj.B*obj.B;
[vec,val] = eig(obj.B2);
val = diag(val);
val(val<1e-4) = 1e-4;
val = diag(val);
obj.B2 = vec*val*vec';
obj.B2 = (obj.B2+obj.B2')/2;
assert(det(obj.B2)>0);

if pIndex
    obj.LM = chol(inv(obj.B2(pIndex,pIndex)));
    obj.PM = obj.LM*data(pIndex,:);
    obj.constantDynamic = -log(obj.samples*sqrt(det(obj.B2(obj.pIndex,obj.pIndex)))*(2*pi)^(length(obj.pIndex)/2)); %constant for marginal p(y), checked
end

obj.LJ = chol(inv(obj.B2));
obj.PJ = obj.LJ*data;
obj.constantStatic = -log(obj.samples*det(obj.B)*(2*pi)^(obj.dim/2)); %constant for joint p(x,y), checked

end