function s = sample(obj,condition)
% Draw a random sample from the KDE model.
%   condition: [] or (length(pIndex),1) double array. The value to condition 
%              on in conditional KDE models, otherwise empty.
% 
% (c)2014 Andreas M. Lehrmann

if isempty(condition)
    c = randi(size(obj.data,2),1); %sample random component
    s = mvnrnd(obj.data(:,c),obj.B2); %sample from that component
else
    %sample component (should be done in log space to avoid zeros)
    cp = cumsum(mvnpdf(repmat(condition',size(obj.data,2),1),obj.data(obj.pIndex,:)',obj.B2(obj.pIndex,obj.pIndex)));
    if cp==zeros(size(cp))
        lDisp('Warning: Choosing component at random, because all weights are 0.')
        c = randi(length(cp));
    else
        cp = [0;cp./cp(end)];
        c = find(cp>rand(1),1)-1;
    end
    
    %sample from that conditional distribution
    g = Gaussian(obj.data(:,c),obj.B2);
    g.conditional(obj.pIndex);
    s = g.sample(condition);
end
    
end